
/* Graphik-Routinen fr LED-Modul EA-DIP122-5NLED */
/* und SED1520 Controller */
/* Copyright (C) 2001  Michael Schnzler */

#include "hardware.h"
#include <inttypes.h>
#include <progmem.h>
#include <stdbool.h>

#define TRUE true
#define FALSE false

void init_lcd (void);

#define lcd_e1_high()    sbi(LCD_E1_PORT, LCD_E1_PIN)
#define lcd_e1_low()     cbi(LCD_E1_PORT, LCD_E1_PIN)

#define lcd_e2_high()    sbi(LCD_E2_PORT, LCD_E2_PIN)
#define lcd_e2_low()     cbi(LCD_E2_PORT, LCD_E2_PIN)

#define lcd_rw_high()    sbi(LCD_RW_PORT, LCD_RW_PIN)
#define lcd_rw_low()     cbi(LCD_RW_PORT, LCD_RW_PIN)

#define lcd_a0_high()    sbi(LCD_A0_PORT, LCD_A0_PIN)
#define lcd_a0_low()     cbi(LCD_A0_PORT, LCD_A0_PIN)


/* SED1520 Controller Primitives */
void display_on (void);
void display_off (void);
void write_register1 (uint8_t data);
void write_register2 (uint8_t data);
uint8_t read_display_data1 (void);
uint8_t read_display_data2 (void);
void write_display_data1 (uint8_t data);
void write_display_data2 (uint8_t data);
void write_display_data (uint8_t data, uint8_t col, uint8_t page,
			 uint8_t (*op)(uint8_t, uint8_t, uint8_t),
			 uint8_t mask);
void set_page (uint8_t page);
void set_column (uint8_t col);
void until_lcd_controller1_ready (void);
void until_lcd_controller2_ready (void);

#define DISPLAY_ON 175
#define DISPLAY_OFF 174
#define SET_PAGE 184
#define SET_COLUMN 0
#define SET_DISPLAY_START_LINE 192
#define DUTY_16 168
#define DUTY_32 169
#define RESET 226

/* Graphic Primitives */
/*                    */

typedef struct 
{
  uint8_t width;
  uint8_t height;
  uint8_t pixel_data[];
} bitmap;

/* typedef struct */
/* { */
/*   uint8_t dwidth; */
/*   uint8_t bbx_width; */
/*   uint8_t bbx_height; */
/*   int8_t bbx_x; */
/*   int8_t bbx_y; */
/*   uint8_t pixel_data[]; */
/* } glyph; */

typedef struct {
  char *name;
  uint8_t width;
  uint8_t height;
  uint8_t PROGMEM *glyphs;
/*     int8_t x_offset; */
/*     int8_t y_offset; */
/*     int8_t ascent; */
/*     int8_t descent; */
} font;

void lcd_drawpixel (uint8_t x, uint8_t y);
void lcd_clearpixel (uint8_t x, uint8_t y);
void lcd_drawline(uint8_t x1, uint8_t y1, uint8_t x2, uint8_t y2);

void lcd_drawbitmap_P (uint8_t x, uint8_t y, bitmap PROGMEM *b);
void lcd_drawbitmap (uint8_t x, uint8_t y, bitmap *b);
void lcd_drawpixeldata (uint8_t x1, uint8_t y1, 
			uint8_t dx, uint8_t dy, uint8_t *pixel_data,
			uint8_t (*operation)(uint8_t, uint8_t, uint8_t));
void lcd_drawpixeldata_P (uint8_t x1, uint8_t y1, 
			  uint8_t dx, uint8_t dy, uint8_t PROGMEM *pixel_data,
			  uint8_t (*operation)(uint8_t, uint8_t, uint8_t));
/* void lcd_drawglyph (uint8_t x, uint8_t y, glyph *g); */

/* void lcd_invert (uint8_t x, uint8_t y, uint8_t dx, uint8_t dy); */
void lcd_cleardisplay (void);

uint8_t lcd_or (uint8_t a, uint8_t b, uint8_t mask);
uint8_t lcd_xor (uint8_t a, uint8_t b, uint8_t mask);
uint8_t lcd_and (uint8_t a, uint8_t b, uint8_t mask);
uint8_t lcd_replace (uint8_t a, uint8_t b, uint8_t mask);
uint8_t lcd_not (uint8_t a, uint8_t b, uint8_t mask);

void print_string (uint8_t x, uint8_t y, char *str);
void lcd_printchar (uint8_t x, uint8_t y, char c);

void lcd_write_char (char c);
void lcd_console_write_char (char c);

uint8_t lcd_cursor_x;
uint8_t lcd_cursor_y;
uint8_t (*lcd_drawing_mode)(uint8_t, uint8_t, uint8_t);
font lcd_current_font;

void init_font5x8(void);
void init_font8x8(void);
void init_font7x13bold(void);
void init_font8x12(void);

#define WIDTH 122
#define HEIGHT 32
#define MAX_X WIDTH - 1
#define MAX_Y HEIGHT - 1
#define MAX_COL 60

