
/* Low-Level Graphik-Routinen fr LED-Modul EA-DIP122-5NLED */
/* und SED1520 Controller */
/* Copyright (C) 2001  Michael Schnzler */

#include <io.h>
#include "lcd_dip122.h"


uint8_t lcd_cursor_x;
uint8_t lcd_cursor_y;
uint8_t (*lcd_drawing_mode)(uint8_t, uint8_t, uint8_t);
void (*write_char)(char);

void init_lcd ()
{
/* configure control lines as output */
  sbi (LCD_E1_PORT_DDR, LCD_E1_PIN);
  sbi (LCD_E2_PORT_DDR, LCD_E2_PIN);
  sbi (LCD_RW_PORT_DDR, LCD_RW_PIN);
  sbi (LCD_A0_PORT_DDR, LCD_A0_PIN);

  lcd_e1_low ();
  lcd_e2_low ();
  lcd_rw_high ();
  lcd_a0_high ();

  write_register1 (RESET);
  write_register2 (RESET);

  outp (0xff, LCD_DATA_PORT_DDR);
  outp (0xff, LCD_DATA_PORT);

  lcd_cursor_x = 0;
  lcd_cursor_y = 31;
  lcd_drawing_mode = lcd_or;
  write_char = lcd_write_char;
}

/* SED1520 Controller Primitives */

void display_on ()
{
  write_register1 (DISPLAY_ON);
  write_register2 (DISPLAY_ON);
}

void display_off ()
{
  write_register1 (DISPLAY_OFF);
  write_register2 (DISPLAY_OFF);
}

void write_register1 (uint8_t data)
{
  until_lcd_controller1_ready ();

  lcd_rw_low ();
  lcd_a0_low ();
  lcd_e1_high ();

  outp (0xFF, LCD_DATA_PORT_DDR);
  outp (data, LCD_DATA_PORT);

  lcd_e1_low ();
}

void write_register2 (uint8_t data)
{
  until_lcd_controller2_ready ();

  lcd_rw_low ();
  lcd_a0_low ();
  lcd_e2_high ();

  outp (0xFF, LCD_DATA_PORT_DDR);
  outp (data, LCD_DATA_PORT);

  lcd_e2_low ();
}

void until_lcd_controller1_ready ()
{
  /* configure data pins as input */
  outp (0xff, LCD_DATA_PORT); /* pull up */
  outp (0x00, LCD_DATA_PORT_DDR);

  lcd_rw_high ();
  lcd_a0_low ();

  lcd_e1_high ();
  asm volatile ("nop" ::);
  loop_until_bit_is_clear (LCD_DATA_PORT_PINS, 7);
  lcd_e1_low ();
}

void until_lcd_controller2_ready ()
{
  /* configure data pins as input */
  outp (0xff, LCD_DATA_PORT); /* pull up */
  outp (0x00, LCD_DATA_PORT_DDR);

  lcd_rw_high ();
  lcd_a0_low ();

  lcd_e2_high ();
  asm volatile ("nop" ::);
  loop_until_bit_is_clear (LCD_DATA_PORT_PINS, 7);
  lcd_e2_low ();
}

void write_display_data1 (uint8_t data)
{
  until_lcd_controller1_ready ();

  lcd_a0_high();
  lcd_rw_low();
  lcd_e1_high();

  outp (0xFF, LCD_DATA_PORT_DDR);
  outp (data, LCD_DATA_PORT);

  lcd_e1_low ();
}

void write_display_data2 (uint8_t data)
{
  until_lcd_controller2_ready ();

  lcd_a0_high();
  lcd_rw_low();
  lcd_e2_high();

  outp (0xFF, LCD_DATA_PORT_DDR);
  outp (data, LCD_DATA_PORT);

  lcd_e2_low ();
}

void write_display_data (uint8_t data, uint8_t col, uint8_t page,
			 uint8_t (*op)(uint8_t, uint8_t, uint8_t),
			 uint8_t mask)
{
  uint8_t s, s2;

  set_page (page);

  if (col <= MAX_COL) {
    set_column(col);
    read_display_data1();             /* dummy         */
    s2 = read_display_data1();
    set_column(col);
    s = (*op)(data, s2, mask);
    write_display_data1(s);
  }
  else {
    set_column(col-MAX_COL-1);
    read_display_data2();
    s2 = read_display_data2();
    set_column(col-MAX_COL-1);
    s = (*op)(data, s2, mask);
    write_display_data2(s);
  }
}

uint8_t read_display_data1 ()
{
  uint8_t data;

  until_lcd_controller1_ready ();

  lcd_a0_high();
  lcd_rw_high();
  lcd_e1_high();

  asm volatile ("nop" ::);
  data = inp (LCD_DATA_PORT_PINS);
  
  lcd_e1_low();

  return data;
}

uint8_t read_display_data2 ()
{
  uint8_t data;

  until_lcd_controller2_ready ();

  lcd_a0_high();
  lcd_rw_high();
  lcd_e2_high();

  asm volatile ("nop" ::);
  data = inp (LCD_DATA_PORT_PINS);
  
  lcd_e2_low();

  return data;
}

void set_page (uint8_t page)
{
  write_register1 (SET_PAGE | (page & 3));
  write_register2 (SET_PAGE | (page & 3));
}

void set_column (uint8_t col)
{
  write_register1 (SET_COLUMN | col);
  write_register2 (SET_COLUMN | col);
}

void set_duty (uint8_t duty)
{
  write_register1 (duty);
  write_register2 (duty);
}

uint8_t lcd_or (uint8_t a, uint8_t b, uint8_t mask  __attribute__ ((unused)))
{
  return a | b;
}

uint8_t lcd_xor (uint8_t a, uint8_t b, uint8_t mask  __attribute__ ((unused)))
{
  return a ^ b;
}

/* uint8_t lcd_and (uint8_t a, uint8_t b) */
/* { */
/*   return a & b; */
/* } */

uint8_t lcd_replace (uint8_t a, uint8_t b, uint8_t mask)
{
  return a | (b & ~mask);
}

uint8_t lcd_not (uint8_t a __attribute__ ((unused)), uint8_t b, uint8_t mask)
{
  return ~(b & mask) | (b & ~mask);
}
